/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (c) 2005,2006,2007 Sven Slezak <sunny@mezzo.net>
 */

/*! \file
 *
 * \brief Charset conversion related dialplan functions
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#include <asterisk/module.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/logger.h>
#include <asterisk/utils.h>
#include <asterisk/app.h>
#include <asterisk/options.h>

#include <ctype.h>
#include <errno.h>
#include <iconv.h>

#define AST_MODULE "func_iconv"

int strconvert(const char *incharset, const char *outcharset,
#ifdef __Darwin__
               const char *in,
#else
               char *in,
#endif
               char *out) {
        iconv_t cd;
        size_t incount = strlen(in), outcount = strlen(in)*2, result;
        cd = iconv_open(outcharset, incharset);
        if(cd == (iconv_t) -1) {
                ast_log(LOG_ERROR, "conversion from '%s' to '%s' not available. type 'iconv -l' in a shell to list the supported charsets.\n", incharset, outcharset);
                memcpy(out, in, strlen(in)+1);
                return -1;
        }
        result = iconv(cd, &in, &incount, &out, &outcount);
        if(result == (size_t)-1) {
                if(errno == E2BIG) {
                        ast_log(LOG_WARNING, "Iconv: output buffer too small.\n");
                } else if(errno == EILSEQ) {
                        ast_log(LOG_WARNING,  "Iconv: illegal character.\n");
                } else if(errno == EINVAL) {
                        ast_log(LOG_WARNING,  "Iconv: incomplete character sequence.\n");
                } else {
                        ast_log(LOG_WARNING,  "Iconv: error.\n");
                }
        }
        iconv_close(cd);
        *out = '\0';
        return 1;
}



static int iconv_read(struct ast_channel *chan, char *cmd, char *parse, char *buf, size_t len)
{
	char *string_converted;
	AST_DECLARE_APP_ARGS(args, AST_APP_ARG(in_charset); AST_APP_ARG(out_charset); AST_APP_ARG(text););

	if (ast_strlen_zero(parse)) {
		ast_log(LOG_WARNING, "Syntax: ICONV(<in-charset>,<out-charset>,<text>) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, parse);

	if (args.argc < 3) {
          ast_log(LOG_WARNING, "Syntax: ICONV(<in-charset>,<out-charset>,<text>) %d\n", args.argc);
		return -1;
	}


  string_converted = alloca(sizeof(char)*strlen(args.text)*4);

  if(option_verbose > 3)
    ast_verbose(VERBOSE_PREFIX_3 "Iconv: \"%s\" %s -> %s\n", args.text, args.in_charset, args.out_charset);

  strconvert(args.in_charset, args.out_charset, args.text, string_converted);

  ast_copy_string(buf, string_converted, len);

	return 0;
}


static struct ast_custom_function iconv_function = {
        .name = "ICONV",
        .synopsis = "Converts charsets of strings.",
        .desc = "Converts string from in-charset into out-charset.\n"
        "for available charsets call 'iconv -l' on your shell command line.\n",
        .syntax = "ICONV(in-charset,out-charset,string)",
        .read = iconv_read,
};

static int unload_module(void)
{
        return ast_custom_function_unregister(&iconv_function);
}

static int load_module(void)
{
        return ast_custom_function_register(&iconv_function);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "String charset conversion dialplan function.");
